@@ -1,5 +1,5 @@
 
-/* Copyright 2018 Richard Kelsch, All Rights Reserved
+/* Copyright 2018-2019 Richard Kelsch, All Rights Reserved
  * See the Perl documentation for Graphics::Framebuffer for licensing information.
  * 
  * Version:  5.00
@@ -149,14 +149,6 @@ void c_plot(
                             *((unsigned short*)(framebuffer + index)) = (short) color;
                         }
                         break;
-                    case 8 :
-                        {
-                        }
-                        break;
-                    case 1 :
-                        {
-                        }
-                        break;
                 }
             break;
             case XOR_MODE :
@@ -223,7 +215,7 @@ void c_plot(
                 }
             break;
             case MASK_MODE :
-                switch(bytes_per_pixel) {
+                switch(bits_per_pixel) {
                     case 32 :
                         {
                             if ((*((unsigned int*)(framebuffer + index )) & 0xFFFFFF00) != (bcolor & 0xFFFFFF00)) { // Ignore alpha channel
@@ -336,7 +328,7 @@ void c_plot(
                 }
             break;
             case ADD_MODE :
-                switch(bytes_per_pixel) {
+                switch(bits_per_pixel) {
                     case 32 :
                         {
                             *((unsigned int*)(framebuffer + index)) += color;
@@ -498,7 +490,6 @@ void c_blit_read(
     short x, short y,
     unsigned short w, unsigned short h,
     unsigned char bytes_per_pixel,
-    unsigned char bits_per_pixel,
     unsigned char draw_mode,
     unsigned char alpha,
     unsigned int bcolor,
@@ -526,16 +517,14 @@ void c_blit_read(
                     unsigned int vhz       = vbl + hzpixel;
                     unsigned int yvhz      = yvbl + hzpixel;
                     unsigned int xhbp_yvbl = xhbp + yvbl;
-                    if (bits_per_pixel == 32) {
+                    if (bytes_per_pixel == 4) {
                         *((unsigned int*)(blit_data + vhz)) = *((unsigned int*)(framebuffer + xhbp_yvbl));
-                    } else if (bits_per_pixel == 24) {
+                    } else if (bytes_per_pixel == 3) {
                         *(blit_data + vhz ) = *(framebuffer + xhbp_yvbl );
                         *(blit_data + vhz  + 1) = *(framebuffer + xhbp_yvbl  + 1);
                         *(blit_data + vhz  + 2) = *(framebuffer + xhbp_yvbl  + 2);
-                    } else if (bits_per_pixel == 16) {
+                    } else {
                         *((unsigned short*)(blit_data + vhz )) = *((unsigned short*)(framebuffer + xhbp_yvbl ));
-                    } else if (bits_per_pixel == 8) {
-                    } else if (bits_per_pixel == 1) {
                     }
                 }
             }
@@ -555,7 +544,6 @@ void c_blit_write(
     short x, short y,
     unsigned short w, unsigned short h,
     unsigned char bytes_per_pixel,
-    unsigned char bits_per_pixel,
     unsigned char draw_mode,
     unsigned char alpha,
     unsigned int bcolor,
@@ -581,8 +569,8 @@ void c_blit_write(
     } else {
         switch(draw_mode) {
             case NORMAL_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -602,7 +590,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -624,7 +612,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -647,8 +635,8 @@ void c_blit_write(
                 }
                 break;
             case XOR_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -668,7 +656,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -690,7 +678,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -713,8 +701,8 @@ void c_blit_write(
                 }
                 break;
             case OR_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -734,7 +722,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -756,7 +744,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -779,8 +767,8 @@ void c_blit_write(
                 }
                 break;
             case AND_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -800,7 +788,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -822,7 +810,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -845,8 +833,8 @@ void c_blit_write(
                 }
                 break;
             case MASK_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -869,7 +857,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -893,7 +881,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -919,8 +907,8 @@ void c_blit_write(
                 }
                 break;
             case UNMASK_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -942,7 +930,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -966,7 +954,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -991,8 +979,8 @@ void c_blit_write(
                 }
                 break;
             case ALPHA_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1029,7 +1017,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1064,7 +1052,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1102,8 +1090,8 @@ void c_blit_write(
                 }
                 break;
             case ADD_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1123,7 +1111,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1145,7 +1133,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1168,8 +1156,8 @@ void c_blit_write(
                 }
                 break;
             case SUBTRACT_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1189,7 +1177,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1211,7 +1199,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1234,8 +1222,8 @@ void c_blit_write(
                 }
                 break;
             case MULTIPLY_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1255,7 +1243,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1277,7 +1265,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1300,8 +1288,8 @@ void c_blit_write(
                 }
                 break;
             case DIVIDE_MODE :
-                switch(bits_per_pixel) {
-                    case 32 :
+                switch(bytes_per_pixel) {
+                    case 4 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1321,7 +1309,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 24 :
+                    case 3 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1343,7 +1331,7 @@ void c_blit_write(
                             }
                         }
                         break;
-                    case 16 :
+                    case 2 :
                         for (vertical = 0; vertical < h; vertical++) {
                             unsigned int vbl  = vertical * bline;
                             unsigned short yv = fb_y + vertical;
@@ -1377,8 +1365,7 @@ void c_rotate(
     unsigned short height,
     unsigned short wh,
     double degrees,
-    unsigned char bytes_per_pixel,
-    unsigned char bits_per_pixel)
+    unsigned char bytes_per_pixel)
 {
     unsigned int hwh        = floor(wh / 2 + 0.5);
     unsigned int bbline     = wh * bytes_per_pixel;
@@ -1410,7 +1397,7 @@ void c_rotate(
                             *(new_img + (x * bytes_per_pixel) + (y * bbline) + 2) = *(image + (xs * bytes_per_pixel) + (ys * bline) + 2);
                         }
                         break;
-                    default :
+                    case 2 :
                         {
                             *((unsigned short*)(new_img + (x * bytes_per_pixel) + (y * bbline))) = *((unsigned short*)(image + (xs * bytes_per_pixel) + (ys * bline)));
                         }
@@ -1460,237 +1447,139 @@ void c_flip_vertical(char *pixels, unsigned short width, unsigned short height,
 }
 
 /* bitmap conversions */
-void c_convert(unsigned char from, unsigned char to, char* buf_from, unsigned size_from, char* buf_to, unsigned char color_order) {
-   switch (from) {
-       case 32 :
-           {
-               switch (to) {
-                   case 24 :
-                       {
-                          unsigned int loc24 = 0;
-                          unsigned int loc32 = 0;
-                          while(loc32 < size_from) {
-                             *(buf_to + loc24++) = *(buf_from + loc32++);
-                             *(buf_to + loc24++) = *(buf_from + loc32++);
-                             *(buf_to + loc24++) = *(buf_from + loc32++);
-                             loc32++; // Toss the alpha
-                          }
-                       }
-                       break;
-                   case 16 :
-                       {
-                          unsigned int loc16    = 0;
-                          unsigned int loc32    = 0;
-                          unsigned short rgb565 = 0;
-                          while(loc32 < size_from) {
-                             unsigned int crgb = *((unsigned int*)(buf_from + loc32));
-                             unsigned char r8 = crgb & 255;
-                             unsigned char g8 = (crgb >> 8) & 255;
-                             unsigned char b8 = (crgb >> 16) & 255;
-                             //        unsigned char a8 = (crgb >> 24) & 255; // This is not used, but is needed
-                             unsigned char r5 = ( r8 * 249 + 1014 ) >> 11;
-                             unsigned char g6 = ( g8 * 253 + 505  ) >> 10;
-                             unsigned char b5 = ( b8 * 249 + 1014 ) >> 11;
-                             if (color_order == 0) {
-                                rgb565 = (b5 << 11) | (g6 << 5) | r5;
-                                *((unsigned short*)(buf_to + loc16)) = rgb565;
-                             } else {
-                                rgb565 = (r5 << 11) | (g6 << 5) | b5;
-                                *((unsigned short*)(buf_to + loc16)) = rgb565;
-                             }
-                             loc16 += 2;
-                          }
-                       }
-                       break;
-                   case 8 :
-                       {
-                       }
-                       break;
-                   case 1 :
-                       {
-                       }
-                       break;
-               }
-           }
-           break;
-       case 24 :
-           {
-               switch (to) {
-                   case 32 :
-                       {
-                          unsigned int loc32 = 0;
-                          unsigned int loc24 = 0;
-                          while(loc24 < size_from) {
-                             unsigned char r = *(buf_from + loc24++);
-                             unsigned char g = *(buf_from + loc24++);
-                             unsigned char b = *(buf_from + loc24++);
-                             *((unsigned int*)(buf_to + loc32++)) = r | (g << 8) | (b << 16);
-                             loc32 += 3;
-                             if (r == 0 && g == 0 && b == 0) {
-                                *(buf_to + loc32++) = 0;
-                             } else {
-                                *(buf_to + loc32++) = 255;
-                             }
-                          }
-                       }
-                       break;
-                   case 16 :
-                       {
-                          unsigned int loc16 = 0;
-                          unsigned int loc24 = 0;
-                          unsigned short rgb565 = 0;
-                          while(loc24 < size_from) {
-                             unsigned char r8 = *(buf_from + loc24++);
-                             unsigned char g8 = *(buf_from + loc24++);
-                             unsigned char b8 = *(buf_from + loc24++);
-                             unsigned char r5 = ( r8 * 249 + 1014 ) >> 11;
-                             unsigned char g6 = ( g8 * 253 + 505  ) >> 10;
-                             unsigned char b5 = ( b8 * 249 + 1014 ) >> 11;
-                             if (color_order == 0) {
-                                rgb565 = (b5 << 11) | (g6 << 5) | r5;
-                                *((unsigned short*)(buf_to + loc16)) = rgb565;
-                             } else {
-                                rgb565 = (r5 << 11) | (g6 << 5) | b5;
-                                *((unsigned short*)(buf_to + loc16)) = rgb565;
-                             }
-                             loc16 += 2;
-                          }
-                       }
-                       break;
-                   case 8 :
-                       {
-                       }
-                       break;
-                   case 1 :
-                       {
-                       }
-                       break;
-               }
-           }
-           break;
-       case 16 :
-           {
-               switch (to) {
-                   case 32 :
-                       {
-                          unsigned int loc16 = 0;
-                          unsigned int loc32 = 0;
-                          unsigned char r5;
-                          unsigned char g6;
-                          unsigned char b5;
+void c_convert_16_24( char* buf16, unsigned int size16, char* buf24, unsigned char color_order ) {
+    unsigned int loc16 = 0;
+    unsigned int loc24 = 0;
+    unsigned char r5;
+    unsigned char g6;
+    unsigned char b5;
 
-                          while(loc16 < size_from) {
-                             unsigned short rgb565 = *((unsigned short*)(buf_from + loc16));
-                             loc16 += 2;
-                             if (color_order == 0) {
-                                b5 = (rgb565 >> 11) & 31;
-                                r5 = rgb565         & 31;
-                             } else {
-                                r5 = (rgb565 >> 11) & 31;
-                                b5 = rgb565         & 31;
-                             }
-                             g6 = (rgb565 >> 5)  & 63;
-                             unsigned char r8 = (r5 * 527 + 23) >> 6;
-                             unsigned char g8 = (g6 * 259 + 33) >> 6;
-                             unsigned char b8 = (b5 * 527 * 23) >> 6;
-                             *((unsigned int*)(buf_to + loc32)) = r8 | (g8 << 8) | (b8 << 16);
-                             loc32 += 3;
-                             if (r8 == 0 && g8 == 0 && b8 ==0) {
-                                *((unsigned char*)(buf_to + loc32++)) = 0;
-                             } else {
-                                *((unsigned char*)(buf_to + loc32++)) = 255;
-                             }
-                          }
-                       }
-                       break;
-                   case 24 :
-                       {
-                          unsigned int loc16 = 0;
-                          unsigned int loc24 = 0;
-                          unsigned char r5;
-                          unsigned char g6;
-                          unsigned char b5;
-
-                          while(loc16 < size_from) {
-                             unsigned short rgb565 = *((unsigned short*)(buf_from + loc16));
-                             loc16 += 2;
-                             if (color_order == 0) {
-                                b5 = (rgb565 >> 11) & 31;
-                                r5 = rgb565         & 31;
-                             } else {
-                                r5 = (rgb565 >> 11) & 31;
-                                b5 = rgb565         & 31;
-                             }
-                             g6 = (rgb565 >> 5)  & 63;
-                             unsigned char r8 = (r5 * 527 + 23) >> 6;
-                             unsigned char g8 = (g6 * 259 + 33) >> 6;
-                             unsigned char b8 = (b5 * 527 * 23) >> 6;
-                             *((unsigned char*)(buf_to + loc24++)) = r8;
-                             *((unsigned char*)(buf_to + loc24++)) = g8;
-                             *((unsigned char*)(buf_to + loc24++)) = b8;
-                          }
-                       }
-                       break;
-                   case 8 :
-                       {
-                       }
-                       break;
-                   case 1 :
-                       {
-                       }
-                       break;
-               }
-           }
-           break;
-       case 8 :
-           {
-               switch (to) {
-                   case 32 :
-                       {
-                       }
-                       break;
-                   case 24 :
-                       {
-                       }
-                       break;
-                   case 16 :
-                       {
-                       }
-                       break;
-                   case 1 :
-                       {
-                       }
-                       break;
-               }
-           }
-           break;
-       case 1 :
-           {
-               switch (to) {
-                   case 32 :
-                       {
-                       }
-                       break;
-                   case 24 :
-                       {
-                       }
-                       break;
-                   case 16 :
-                       {
-                       }
-                       break;
-                   case 8 :
-                       {
-                       }
-                       break;
-               }
-           }
-           break;
-   }
-   
+    while(loc16 < size16) {
+        unsigned short rgb565 = *((unsigned short*)(buf16 + loc16));
+        loc16 += 2;
+        if (color_order == 0) {
+            b5 = (rgb565 >> 11) & 31;
+            r5 = rgb565         & 31;
+        } else {
+            r5 = (rgb565 >> 11) & 31;
+            b5 = rgb565         & 31;
+        }
+        g6 = (rgb565 >> 5)  & 63;
+        unsigned char r8 = (r5 * 527 + 23) >> 6;
+        unsigned char g8 = (g6 * 259 + 33) >> 6;
+        unsigned char b8 = (b5 * 527 * 23) >> 6;
+        *((unsigned char*)(buf24 + loc24++)) = r8;
+        *((unsigned char*)(buf24 + loc24++)) = g8;
+        *((unsigned char*)(buf24 + loc24++)) = b8;
+    }
 }
 
-void c_monochrome(char *pixels, unsigned int size, unsigned short color_order, unsigned char bytes_per_pixel, unsigned char bits_per_pixel) {
+void c_convert_16_32( char* buf16, unsigned int size16, char* buf32, unsigned char color_order ) {
+    unsigned int loc16 = 0;
+    unsigned int loc32 = 0;
+    unsigned char r5;
+    unsigned char g6;
+    unsigned char b5;
+
+    while(loc16 < size16) {
+        unsigned short rgb565 = *((unsigned short*)(buf16 + loc16));
+        loc16 += 2;
+        if (color_order == 0) {
+            b5 = (rgb565 >> 11) & 31;
+            r5 = rgb565         & 31;
+        } else {
+            r5 = (rgb565 >> 11) & 31;
+            b5 = rgb565         & 31;
+        }
+        g6 = (rgb565 >> 5)  & 63;
+        unsigned char r8 = (r5 * 527 + 23) >> 6;
+        unsigned char g8 = (g6 * 259 + 33) >> 6;
+        unsigned char b8 = (b5 * 527 * 23) >> 6;
+        *((unsigned int*)(buf32 + loc32)) = r8 | (g8 << 8) | (b8 << 16);
+        loc32 += 3;
+        if (r8 == 0 && g8 == 0 && b8 ==0) {
+            *((unsigned char*)(buf32 + loc32++)) = 0;
+        } else {
+            *((unsigned char*)(buf32 + loc32++)) = 255;
+        }
+    }
+}
+
+void c_convert_24_16(char* buf24, unsigned int size24, char* buf16, unsigned char color_order) {
+    unsigned int loc16 = 0;
+    unsigned int loc24 = 0;
+    unsigned short rgb565 = 0;
+    while(loc24 < size24) {
+        unsigned char r8 = *(buf24 + loc24++);
+        unsigned char g8 = *(buf24 + loc24++);
+        unsigned char b8 = *(buf24 + loc24++);
+        unsigned char r5 = ( r8 * 249 + 1014 ) >> 11;
+        unsigned char g6 = ( g8 * 253 + 505  ) >> 10;
+        unsigned char b5 = ( b8 * 249 + 1014 ) >> 11;
+        if (color_order == 0) {
+            rgb565 = (b5 << 11) | (g6 << 5) | r5;
+            *((unsigned short*)(buf16 + loc16)) = rgb565;
+        } else {
+            rgb565 = (r5 << 11) | (g6 << 5) | b5;
+            *((unsigned short*)(buf16 + loc16)) = rgb565;
+        }
+        loc16 += 2;
+    }
+}
+
+void c_convert_32_16(char* buf32, unsigned int size32, char* buf16, unsigned char color_order) {
+    unsigned int loc16    = 0;
+    unsigned int loc32    = 0;
+    unsigned short rgb565 = 0;
+    while(loc32 < size32) {
+        unsigned int crgb = *((unsigned int*)(buf32 + loc32));
+        unsigned char r8 = crgb & 255;
+        unsigned char g8 = (crgb >> 8) & 255;
+        unsigned char b8 = (crgb >> 16) & 255;
+//        unsigned char a8 = (crgb >> 24) & 255; // This is not used, but is needed
+        unsigned char r5 = ( r8 * 249 + 1014 ) >> 11;
+        unsigned char g6 = ( g8 * 253 + 505  ) >> 10;
+        unsigned char b5 = ( b8 * 249 + 1014 ) >> 11;
+        if (color_order == 0) {
+            rgb565 = (b5 << 11) | (g6 << 5) | r5;
+            *((unsigned short*)(buf16 + loc16)) = rgb565;
+        } else {
+            rgb565 = (r5 << 11) | (g6 << 5) | b5;
+            *((unsigned short*)(buf16 + loc16)) = rgb565;
+        }
+        loc16 += 2;
+    }
+}
+
+void c_convert_32_24(char* buf32, unsigned int size32, char* buf24, unsigned char color_order) {
+    unsigned int loc24 = 0;
+    unsigned int loc32 = 0;
+    while(loc32 < size32) {
+        *(buf24 + loc24++) = *(buf32 + loc32++);
+        *(buf24 + loc24++) = *(buf32 + loc32++);
+        *(buf24 + loc24++) = *(buf32 + loc32++);
+        loc32++; // Toss the alpha
+    }
+}
+
+void c_convert_24_32(char* buf24, unsigned int size24, char* buf32, unsigned char color_order) {
+    unsigned int loc32 = 0;
+    unsigned int loc24 = 0;
+    while(loc24 < size24) {
+        unsigned char r = *(buf24 + loc24++);
+        unsigned char g = *(buf24 + loc24++);
+        unsigned char b = *(buf24 + loc24++);
+        *((unsigned int*)(buf32 + loc32++)) = r | (g << 8) | (b << 16);
+        loc32 += 3;
+        if (r == 0 && g == 0 && b == 0) {
+            *(buf32 + loc32++) = 0;
+        } else {
+            *(buf32 + loc32++) = 255;
+        }
+    }
+}
+
+void c_monochrome(char *pixels, unsigned int size, unsigned short color_order, unsigned char bytes_per_pixel) {
     unsigned int idx;
     unsigned char r;
     unsigned char g;
@@ -1699,7 +1588,7 @@ void c_monochrome(char *pixels, unsigned int size, unsigned short color_order, u
     unsigned short rgb565;
 
     for (idx = 0; idx < size; idx += bytes_per_pixel) {
-        if (bits_per_pixel >= 24) {
+        if (bytes_per_pixel >= 3) {
             switch(color_order) {
                 case RBG :  // RBG
                     r = *(pixels + idx);
@@ -1731,7 +1620,7 @@ void c_monochrome(char *pixels, unsigned int size, unsigned short color_order, u
                     g = *(pixels + idx + 1);
                     b = *(pixels + idx + 2);
             }
-        } else if (bits_per_pixel == 16) {
+        } else {
             rgb565 = *((unsigned short*)(pixels + idx));
             g      = (rgb565 >> 6) & 31;
             if (color_order == 0) { // RGB
@@ -1741,123 +1630,20 @@ void c_monochrome(char *pixels, unsigned int size, unsigned short color_order, u
                 b = rgb565 & 31;
                 r = (rgb565 >> 11) & 31;
             }
-        } else if (bits_per_pixel == 8) {
-        } else if (bits_per_pixel == 1) {
         }
         m = (unsigned char) round(0.2126 * r + 0.7152 * g + 0.0722 * b);
 
-        if (bits_per_pixel >= 24) {
+        if (bytes_per_pixel >= 3) {
             *(pixels + idx)     = m;
             *(pixels + idx + 1) = m;
             *(pixels + idx + 2) = m;
-        } else if (bits_per_pixel == 16) {
+        } else { // 2
             rgb565                             = 0;
             rgb565                             = (m << 11) | (m << 6) | m;
             *((unsigned short*)(pixels + idx)) = rgb565;
-        } else if (bits_per_pixel == 8) {
-        } else if (bits_per_pixel == 1) {
         }
     }
 }
 
-/* Antialiased line drawing 
-   NOT READY YET
- 
-// draws a pixel on screen of given brightness
-// 0<=brightness<=1. We can use your own library
-// to draw on screen
-void drawPixel( int x , int y , float brightness) {
-    int c = 255*brightness;
-    SDL_SetRenderDrawColor(pRenderer, c, c, c, 255);
-    SDL_RenderDrawPoint(pRenderer, x, y);
-}
- 
-void drawAALine(int x0 , int y0 , int x1 , int y1) {
-    int steep = absolute(y1 - y0) > absolute(x1 - x0) ;
- 
-    // swap the co-ordinates if slope > 1 or we
-    // draw backwards
-    if (steep) {
-        swap(&x0 , &y0);
-        swap(&x1 , &y1);
-    }
-    if (x0 > x1) {
-        swap(&x0 ,&x1);
-        swap(&y0 ,&y1);
-    }
- 
-    //compute the slope
-    float dx = x1-x0;
-    float dy = y1-y0;
-    float gradient = dy/dx;
-    if (dx == 0.0)
-        gradient = 1;
- 
-    int xpxl1 = x0;
-    int xpxl2 = x1;
-    float intersectY = y0;
- 
-    // main loop
-    if (steep) {
-        int x;
-        for (x = xpxl1 ; x <=xpxl2 ; x++) {
-            // pixel coverage is determined by fractional
-            // part of y co-ordinate
-            drawPixel(iPartOfNumber(intersectY), x,
-                        rfPartOfNumber(intersectY));
-            drawPixel(iPartOfNumber(intersectY)-1, x,
-                        fPartOfNumber(intersectY));
-            intersectY += gradient;
-        }
-    } else {
-        int x;
-        for (x = xpxl1 ; x <=xpxl2 ; x++) {
-            // pixel coverage is determined by fractional
-            // part of y co-ordinate
-            drawPixel(x, iPartOfNumber(intersectY),
-                        rfPartOfNumber(intersectY));
-            drawPixel(x, iPartOfNumber(intersectY)-1,
-                          fPartOfNumber(intersectY));
-            intersectY += gradient;
-        }
-    }
- 
-}
-
-// swaps two numbers
-void swap(int* a , int*b) {
-    int temp = *a;
-    *a = *b;
-    *b = temp;
-}
- 
-// returns absolute value of number
-float absolute(float x ) {
-    if (x < 0) return -x;
-    else return x;
-}
- 
-//returns integer part of a floating point number
-int iPartOfNumber(float x) {
-    return (int)x;
-}
- 
-//rounds off a number
-int roundNumber(float x) {
-    return iPartOfNumber(x + 0.5) ;
-}
- 
-//returns fractional part of a number
-float fPartOfNumber(float x) {
-    if (x>0) return x - iPartOfNumber(x);
-    else return x - (iPartOfNumber(x)+1); 
-}
- 
-//returns 1 - fractional part of number
-float rfPartOfNumber(float x) {
-    return 1 - fPartOfNumber(x);
-}
-
-*/
 
 C_CODE
